@@ -1,9 +1,9 @@
-from dataclasses import dataclass
+from __future__ import annotations
+
 from enum import Enum
 
+
 # Adapted from prompt toolkit https://github.com/prompt-toolkit/python-prompt-toolkit/blob/master/prompt_toolkit/keys.py
-
-
 class Keys(str, Enum):
     """
     List of keys for use in key bindings.
@@ -16,6 +16,7 @@ class Keys(str, Enum):
 
     Escape = "escape"  # Also Control-[
     ShiftEscape = "shift+escape"
+    Return = "return"
 
     ControlAt = "ctrl+@"  # Also Control-Space.
 
@@ -176,20 +177,16 @@ class Keys(str, Enum):
     ScrollUp = "<scroll-up>"
     ScrollDown = "<scroll-down>"
 
-    CPRResponse = "<cursor-position-response>"
-    Vt100MouseEvent = "<vt100-mouse-event>"
-    WindowsMouseEvent = "<windowshift+mouse-event>"
-    BracketedPaste = "<bracketed-paste>"
-
     # For internal use: key which is ignored.
     # (The key binding for this key should not do anything.)
     Ignore = "<ignore>"
 
     # Some 'Key' aliases (for backwardshift+compatibility).
-    ControlSpace = ControlAt
-    Tab = ControlI
-    Enter = ControlM
-    Backspace = ControlH
+    ControlSpace = "ctrl-at"
+    Tab = "tab"
+    Space = "space"
+    Enter = "enter"
+    Backspace = "backspace"
 
     # ShiftControl was renamed to ControlShift in
     # 888fcb6fa4efea0de8333177e1bbc792f3ff3c24 (20 Feb 2020).
@@ -199,8 +196,30 @@ class Keys(str, Enum):
     ShiftControlEnd = ControlShiftEnd
 
 
-@dataclass
-class Binding:
-    action: str
-    description: str
-    show: bool = False
+# Unicode db contains some obscure names
+# This mapping replaces them with more common terms
+KEY_NAME_REPLACEMENTS = {
+    "solidus": "slash",
+    "reverse_solidus": "backslash",
+    "commercial_at": "at",
+    "hyphen_minus": "minus",
+    "plus_sign": "plus",
+    "low_line": "underscore",
+}
+REPLACED_KEYS = {value: key for key, value in KEY_NAME_REPLACEMENTS.items()}
+
+# Some keys have aliases. For example, if you press `ctrl+m` on your keyboard,
+# it's treated the same way as if you press `enter`. Key handlers `key_ctrl_m` and
+# `key_enter` are both valid in this case.
+KEY_ALIASES = {
+    "tab": ["ctrl+i"],
+    "enter": ["ctrl+m"],
+    "escape": ["ctrl+left_square_brace"],
+    "ctrl+at": ["ctrl+space"],
+    "ctrl+j": ["newline"],
+}
+
+
+def _get_key_aliases(key: str) -> list[str]:
+    """Return all aliases for the given key, including the key itself"""
+    return [key] + KEY_ALIASES.get(key, [])
